// ------------------------------------------------------------------
// (c) Teradyne, Inc, 2006
//     All Rights Reserved
// Inclusion of a copyright notice does not imply that this software
// has been published.  This software is the trade secret information
// of Teradyne, Inc.  Use of this software is only in accordance with
// the terms of a license agreement from Teradyne, Inc.
// ------------------------------------------------------------------
// CompositeUndoContext.cs - Represents a group of undo information and undoes or redoes
//   the whole operation at a single shot
// *************************************************************************************/
//   Revision history:
//   Author         Date        Description of change
//   =================================================================
//   shoban         9/4/2007    Created
// *************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace Teradyne.Igxl.Utilities
{
    class CompositeUndoContext : IUndoContext
    {
        private IUndoContext m_parent;
        private List<IUndoContext> m_contexts;

        /// <summary>
        /// Represents a context which can hold multiple similar contexts. Used
        /// where multiple undo operations require to be grouped together.
        /// </summary>
        /// <param name="parent">The parent context if available. null is a
        /// valid argument if there are no parents</param>
        public CompositeUndoContext(IUndoContext parent)
        {
            m_parent = parent;
            m_contexts = new List<IUndoContext>();
        }

        /// <summary>
        /// Adds a new context item
        /// </summary>
        /// <param name="context">The undo information that needs to be added in 
        /// this context</param>
        public void Add(IUndoContext context)
        {
            m_contexts.Add(context);
        }

        #region IUndoContext Members

        /// <summary>
        /// Updates the state of the system.
        /// </summary>
        /// <param name="isUndoing">Indicates if the operation is an undo or redo</param>
        /// <returns>true on success</returns>
        public bool SetState(bool isUndoing)
        {
            if (isUndoing)
            {
                // Apply the changes in reverse
                int c = m_contexts.Count;
                while (c != 0)
                {
                    if (!m_contexts[--c].SetState(isUndoing))
                        return false;
                }
            }
            else
            {
                foreach (IUndoContext context in m_contexts)
                {
                    if (!context.SetState(isUndoing))
                        return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Returns the parent context
        /// </summary>
        public IUndoContext Parent
        {
            get { return m_parent; }
        }

        /// <summary>
        /// Returns the number of undo information maintained
        /// </summary>
        public int Count
        {
            get { return m_contexts.Count; }
        }

        #endregion
    }
}
