
/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------------------
 *  The NspcpItem class provides a blueprint for carrying all the pertinent
 *  information for a PSet/Signal definition and the respective methods on them. 
 * ------------------------------------------------------------------------------
 */
/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *  Nadeem Khan     8/12        Created.
 ****************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using Teradyne.Igxl.PSetSignalsEditor;
using Teradyne.Igxl.PSetSignalsEditor.Exceptions;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    public class NspcpItem
    {
        private GlobalUtils.NspcpType type;
        private string Name; //Name of this PSet/Signal
        private string PinSpec; //Name of PinSpec associated with this PSet/Signal
        private string LogicalInstrument; //Name of the instrument associated with this PSet/Signal
        private string PhysicalInstrument; //Name of physical instrument for associated logical instrument
        private long ParamNum; //Number of parameters associated with the pin
        public Dictionary<string, string> NameValuePairs;   //List of name value pairs of the PSet/Signal Definitions
        private List<String> errName;       //Name of the errors occurring during validation.
        private List<String> errInfo;       //Information regarding the errors occuring during the validation.

        #region member methods

    
        /// <summary>
        /// Parameterized Constructor for NSPCP Item Class
        /// </summary>
        /// <param name="t"></param>
        /// <param name="name"></param>
        /// <param name="pin"></param>
        /// <param name="li"></param>
        /// <param name="pi"></param>
        /// <param name="pnum"></param>
        /// <param name="pnames"></param>
        /// <param name="pval"></param>
        public NspcpItem(GlobalUtils.NspcpType t, string name, String pin, String li, String pi, 
            long pnum, Dictionary<string, string> nvp)
        {
            this.type = t;
            this.Name = name;
            this.PinSpec = pin;
            this.LogicalInstrument = li;
            this.PhysicalInstrument = pi;
            this.ParamNum = pnum;
            this.NameValuePairs = nvp;
            this.errName = new List<string>();
            this.errInfo = new List<string>();    
        }
        
        /// <summary>
        /// This property sets the Pin Name
        /// </summary>
        public String pinName
        {
            get { return this.PinSpec; }
            set { this.PinSpec = value; }
        }


        /// <summary>
        /// This property sets the error info
        /// </summary>
        public List<String> propErrInfo
        {
            get
            {
                return this.errInfo; 
            }
            set 
            {
                this.errInfo.Clear();
                this.errInfo = value; 
            }
        }


        /// <summary>
        /// This property sets the error name.
        /// </summary>
        public List<String> propErrName
        {
            get 
            { 
                return this.errName; 
            }
            set 
            {
                this.errName.Clear();
                this.errName = value; 
            }
        }


        /// <summary>
        /// This property sets the logical instrument name.
        /// </summary>
        public String propLogicalInstrument
        {
            get { return this.LogicalInstrument; }
            set { this.LogicalInstrument = value; }
        }


        /// <summary>
        /// This property sets the physical instrument (board) name.
        /// </summary>
        public String propPhysicalInstrument
        {
            get { return this.PhysicalInstrument; }
            set { this.PhysicalInstrument = value; }
        }


        /// <summary>
        /// This property sets the nspcp name.
        /// </summary>
        public String propName
        {
            get { return this.Name;}
            set { this.Name = value; }
        }

        /// <summary>
        /// This property sets the nspcp type.
        /// </summary>
        public GlobalUtils.NspcpType propType
        {
            get {return type;}
            set {type = value;}
        }

   
        /// <summary>
        /// Creates and returns a duplicate of the current nspcp object
        /// </summary>
        /// <param name="name">The new name that this duplicated object will be given.</param>
        public NspcpItem Duplicate(string name)
        {
            Dictionary<String, String> newDic = new Dictionary<String, String>();

            //Duplicate Parameter Name and Parameter Value lists.
            foreach (KeyValuePair<String, string> kvp in this.NameValuePairs)
            {
                newDic.Add(kvp.Key, kvp.Value);
            }

            NspcpItem dupObj = new NspcpItem(this.type, name, this.PinSpec, this.LogicalInstrument, this.PhysicalInstrument, this.ParamNum, newDic);
            
            return dupObj;
        }

        /// <summary>
        /// This methods compares whether this nspcp object is same as the
        /// one passed as an argument. 
        /// </summary>
        /// <param name="obj">PSet/Signal object being compared against</param>
        /// <returns>returns true if comparison is true</returns>
        public Boolean Compare(NspcpItem obj)
        {
            Boolean equal;
            equal = true;
            //Check if we are comparing NSPCPs of same type.
            if ((obj.propType == this.propType) && (obj.LogicalInstrument == this.LogicalInstrument))
            {
                foreach (KeyValuePair<string, string> kvp in this.NameValuePairs)
                {
                    if (this.NameValuePairs[kvp.Key] != obj.NameValuePairs[kvp.Key]) equal = false;
                }
            }
            else
            {
                throw new IncompatibleNSPCPException("Incompatible PSet/Signal types");   //Throw an exception incase nspcp types are incompatible.
            }
            return equal;
        }

        #endregion
    }
}
