/* File: SelectionOverride.cs
 * 
 * Revision History:
 * 16 May 07  Stimson  Allow for multiple selections within the 'using' statement.
 *  1 May 07  Stimson  Create.
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    /// <summary>
    /// This class implements temporary override of the current PinInfo selection.
    /// It must be used in the context of a 'using' statement.
    /// </summary>
    /// <remarks>
    /// The use model for this class allows for multiple selections to be made within the 'using' statement.
    /// The constructor call need not establish an initial selection. It is guaranteed that the original selection
    /// will be restored at the end of the 'using' statement. The two identified use cases are:
    /// 1. using (SelectionOverride&lt;D&gt; selection = new SelectionOverride&lt;D&gt;(group)) {
    ///      doSomething(); // execute in the context of group.
    ///    }
    /// 2. using (SelectionOverride&lt;D&gt; selection = new SelectionOverride&lt;D&gt;()) {
    ///      foreach (item in collection) {
    ///        selection.Selection(item.Group);
    ///        doSomething(); // execute in the context of item.Group.
    ///      }
    ///    }
    /// </remarks>
   
    public sealed class SelectionOverride : IDisposable
    {
        private Selection oldSelection;

        /// <summary>
        /// Initialize this object with the selection in place at the point of execution.
        /// The caller is responsible for calling Select within the 'using' statement to
        /// set the new selection.
        /// </summary>
        public SelectionOverride()
        {
            oldSelection = SelectionManager.Current;
        }
        /// <summary>
        /// Initialize this object with a group of PinInfo elements.
        /// Selection is set accordingly.
        /// </summary>
        /// <param name="toSelect">The new temporary selection.</param>
        public SelectionOverride(IGroup<Element<D>> toSelect)
        {
            oldSelection = SelectionManager<D>.Current;
            Select(toSelect);
        }
        /// <summary>
        /// Initialize this object with a single PinInfo element.
        /// Selection is set accordingly.
        /// </summary>
        /// <param name="toSelect">The new temporary selection.</param>
        public SelectionOverride(Element<D> toSelect)
        {
            oldSelection = SelectionManager<D>.Current;
            Select(toSelect);
        }

        /// <summary>
        /// Select the specified group of elements.
        /// </summary>
        /// <param name="toSelect"></param>
        public void Select(IGroup<Element<D>> toSelect)
        {
            bool considerSelectedSites = (oldSelection == null ? true : oldSelection.ConsiderSelectedSites);
            SelectionManager<D>.Select(new Selection<D>(toSelect, considerSelectedSites));
        }
        /// <summary>
        /// Select the specified element.
        /// </summary>
        /// <param name="toSelect"></param>
        public void Select(Element<D> toSelect)
        {
            Group<Element<D>> group = new Group<Element<D>>();
            group.Add(toSelect);
            Select(group);
        }

        #region IDisposable Members
        /// <summary>
        /// Restore selection.
        /// </summary>
        public void Dispose()
        {
            SelectionManager<D>.Current = oldSelection;
        }

        #endregion
    }
}
