
/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------
 * SignalsSheetEngine.cs file acts as an interface through which we can
 * perform various operations on Signal Sheet like Reading data from 
 * sheet, Updating definition of a Signal and getting the List of signals.
 * ------------------------------------------------------------------
 */

/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *   Grace Silvia   01/23/12    tersw00182305 - Added a condition check to UpdateSheet() to set formula to null, if there is only content value.
 *   Venkatachalam  09/27/11    Fixed tersw00180364 - Updated the Myservices.cs.
 *   Kalaivani C D  03/26/10    tersw00153690 - Fix for Signals sheet update from the editor when there is a Joblist sheet in the workbook.
 *   M. Raykin      10/30/07    Set "ScreenUpdateEnabled" property (IDTSheetSvcWorkbook) to false before updating DT data structures in UpdateSheet()
 *                              Reset it back to true at the end of the routine
 *   M. Raykin      10/24/07   Fixed tersw00113179
 *   Maria Raykin   10/19/07   Fixed bug in UpdateSheet() that caused an Assert in Signals sheet while deleting a row 
 *                             that has the same Signal name/different pin and if that row is creted from within Editor
 *   Maria Raykin   10/19/07   Fixed UpdateSheet() bug (updating existing Signal would write data beyond 
 *                             last parameter column).  No need to create new DMPsetspsetpinargs elements, but
 *                             update existing)
 *   Kiran Kumar    10/12/07   Fixed the defect : tersw00112494
 *   Kiran Kumar    09/26/07   Fixed the defect : tersw00111856
 *   Kiran Kumar    09/22/07   Fixed a defect : tersw00111764
 *   Kiran kumar    09/21/07   changed return of case insensitive to Case sensitive.
 *   kiran kumar    09/18/07   Fixed the defect No: tersw00111445
 *   Kiran kumar    08/16/07   Addressed Review comments
 *   Kiran kumar    08/01/07   Created
 ***************************************************************************************
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Teradyne.Igxl.PSetSignalsEditor;
using Teradyne.Igxl.PSetSignalsEditor.PSetSignalsEditorServices;
using DMGRSIGNALSSHEETLIB;
using DTSHEETSERVICELib;
using CollectionEditHelperLib;
using DMGRJOBLISTSHEETLIB;
using DMGRLib;


namespace Teradyne.Igxl.PSetSignalsEditor
{
    public class SignalsSheetEngine : ISheetEngine
    {
        bool is_disposed = false;
        private const int LIST_OF_ALL_PARAMS = 1;
        private const int FIRST_ARG_IN_SHEET = 2;
        private const int FIRST_PIN_NAME = 1;
        private const String SIGNAL_TYPE = "Signal Type";
        private const String SIGNAL_WAVE_DEFINITION_TYPE = "Wave Def or Data";
        private const int FIRST_SIGNAL = 1;
        private const int FIRST_SIGNAL_NAME = 0;
        CCollectionEditHelper m_signalHelper;

        public SignalsSheetEngine(ref CollectionEditHelperLib.CCollectionEditHelper pHelper)
        {
            try
            {
                m_signalHelper = pHelper;
            }
            catch (Exception e)
            {
                Console.Write("Exception caught in SignalsSheetEngine {0}", e.StackTrace);
            }

        }

        protected virtual void Dispose(bool disposing)
        {
            if (!is_disposed)
            {
                if (disposing)
                {
                    m_signalHelper = null;
                }
            }
            this.is_disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            //tell the garbage collector not to finalize
            GC.SuppressFinalize(this);
        }


        ~SignalsSheetEngine()
        {
            Dispose(false);
        }

        #region SheetEngine Members


        /// <summary>
        /// The UpdateSheet() Method is used to update the Signal's definition in the signal sheet.
        /// The parameter NspcpItem object contains the required data to update the sheet.
        /// While updating the sheet, if the sheet does not exist, this method will create one new sheet 
        /// with the specified name and the definition of a signal will be updated in the sheet as new entry.
        /// The update sheet method will check, whether the signal name(which we are passing through 
        /// NspcpItem object) is matching with any of the signals names in the sheet. If it matches, 
        /// then it will check whether Pin name, which we are passing is matching with the  name of the Pin 
        /// associated to the existing signal in the sheet. If it matches then it will check for Instrument
        /// name matching. If it also matches, then the argument values of the existing signal
        /// will be updated. If Instrument Name or Pin name are not matching, then the new entry for the
        /// existing Signal will be created. If Signal name is not matching, then new Signal will be
        /// created and the definition will be updated for the created signal.
        /// </summary>
        /// <param name="obj">This parameter is of type NspcpItem. Which contains the information 
        /// of Signal Name, Pin Name, Instrument name, and Dictionary(Data Collection) of 
        /// Argument names and it's values as Key and Value pairs.</param>
        public void UpdateSheet(NspcpItem nspcpObj)
        {
            //Declare signalSheets to get the list of signal sheets available
            IDMSheets signalSheets;

            DMSignalsSheet signalSheet = null;
            DMSignalssignals signalInfo;
            DMSignalssignalspinargs argssInfo;
            DMSignalssignalspin pinInfo = null;
            char[] delim = { ',' };
            String signalNamesstr;
            String pinNamesStr;
            String[] signalNames;
            String[] pinNames;
            Boolean isPinExist = false;

            int argColumn = FIRST_ARG_IN_SHEET; // Signal definitions start from the second parameter. 
            // The first parameter is a consolidation of all parameter names

            if ((nspcpObj == null) || (nspcpObj.propName == null) || (nspcpObj.pinName == null))
                throw new ArgumentNullException(" obj or obj.propName or obj.propPinSpec are null");

            try
            {
                //// following line will throw an exception if there is no signal sheet available
                //signalSheet = (DMSignalsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                signalSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                if ((signalSheets == null) || (signalSheets.Count == 0)) throw new ArgumentException();

                foreach (DMGRLib.IDMSheet item in signalSheets)
                {
                    DMSignalsSheet signalSheetTemp = item as DMSignalsSheet;                
                    if (signalSheetTemp != null)
                    {
                        signalSheet = signalSheetTemp;
                        break;
                    }
                }

                if (signalSheet == null) throw new Exception("Invalid Signal sheet, the new Signals sheet created is not an active sheet");

            }
            catch (ArgumentException)
            {
                //Create New signal Sheet with the default name.
                MyServices.DTSheetSvcCOM.CreateSheetFromDMGRType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET, null);
                try
                {
                    // following line will throw an exception if there is a JobList sheet in the workbook.
                    signalSheet = (DMSignalsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                }
                catch (ArgumentException)
                {
                    DMJobList jobListSheet;
                    // get the job list sheet element
                    jobListSheet = (DMJobList)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_JOBLISTSHEET);

                    IDTSheetSvcWorkbook wb = (IDTSheetSvcWorkbook)MyServices.DTSheetSvcCOM;
                    IExcelSheet signalsShtPtr;
                    // get the newly created Signals sheet pointer. 
                    wb.GetActiveSheet(out signalsShtPtr);

                    if (jobListSheet.Jobs.Count == 0)
                    {
                        // Invalid Job List sheet.
                        throw new Exception("Invalid JobList sheet, the new Signals sheet created is not an active sheet");
                    }

                    string activeJobNameStr;
                    // get the active job
                    activeJobNameStr = MyServices.DmgrServiceCOM.ContextMgr.JobState.ActiveJobName;

                    IDMJobListJob activeJobInfo;
                    // get the DM element of the active job.
                    activeJobInfo = jobListSheet.Jobs.Find(activeJobNameStr);

                    // add the newly created Signals sheet to the active Job
                    activeJobInfo.Psets = signalsShtPtr.name;
                    signalsShtPtr.Activate();

                    // get the DM element of the active Signals sheet to update the sheet parameters.
                    signalSheet = (DMSignalsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                }
            }
            signalNamesstr = signalSheet.Signalss.Names.CSL;
            signalNames = signalNamesstr.Split(delim);
            for (int parse = FIRST_SIGNAL_NAME; parse < signalNames.Length; parse++)
            {
                if (nspcpObj.propName.ToLower().Equals(signalNames[parse].ToLower()))
                {
                    nspcpObj.propName = signalNames[parse];
                    break;
                }
            }
            String key = "";
            Boolean updateState = true;
            IDTSheetSvcWorkbook workbook = (IDTSheetSvcWorkbook)(MyServices.DTSheetSvcCOM);
            // Set "ScreenUpdateEnabled" property to false in order to sheet reduce flickering
            try
            {
                if (workbook != null)
                {
                    updateState = workbook.ScreenUpdateEnabled;
                    workbook.ScreenUpdateEnabled = false;
                }

                if (signalSheet.Signalss.Has(nspcpObj.propName))
                {
                    // update an existing signal
                    signalInfo = signalSheet.Signalss.Find(nspcpObj.propName);
                    pinNamesStr = signalInfo.Pins.Names.CSL;
                    pinNames = pinNamesStr.Split(delim);

                    String sheetPinList = String.Join(",", DecomposePinList(pinNamesStr)).ToLower(); ;
                    PARAMETERCOLLECTIONINTERFACELib.ITCOStrings sheetPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                    sheetPins.set_TSL(",", sheetPinList);
                    pinNames = pinNamesStr.Split(delim); //the pin names selected in the editor.

                    for (int parse = FIRST_SIGNAL_NAME; parse < pinNames.Length; parse++)
                    {
                        if (nspcpObj.pinName.ToLower().Equals(pinNames[parse].ToLower()))
                        {
                            nspcpObj.pinName = pinNames[parse];
                            break;
                        }
                    }

                    String inputPinList = String.Join(",", DecomposePinList(nspcpObj.pinName)).ToLower(); ;
                    PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                    inputPins.set_TSL(",", inputPinList);

                    if (signalInfo.Pins.Has(nspcpObj.pinName))
                    {
                        isPinExist = true;
                        pinInfo = signalInfo.Pins.Find(nspcpObj.pinName);
                    }
                    else if (signalInfo.Pins.Has(inputPins.CSL))
                    {
                        isPinExist = true;
                        pinInfo = signalInfo.Pins.Find(inputPins.CSL);
                    }
                    else if (sheetPins.CSL.ToLower().Equals(inputPins.CSL.ToLower()))
                    {
                        isPinExist = true;
                        pinInfo = signalInfo.Pins.Find(pinNamesStr);
                    }

                    if (isPinExist)
                    {
                        if (nspcpObj.propLogicalInstrument.ToLower().Equals(pinInfo.Insttype.ToLower()))
                        {
                            pinInfo.Wavedef = nspcpObj.NameValuePairs[SIGNAL_WAVE_DEFINITION_TYPE];
                            pinInfo.Sigtype = nspcpObj.NameValuePairs[SIGNAL_TYPE];
                            foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                            {
                                if (kvp.Key.Equals(SIGNAL_TYPE) || kvp.Key.Equals(SIGNAL_WAVE_DEFINITION_TYPE))
                                    continue;
                                argssInfo = pinInfo.Argss[argColumn];
                                if (argssInfo != null)
                                {
                                    // Update values that have been changed only
                                    if (argssInfo.Name != kvp.Value)
                                    {
                                        object hasformula;
                                        hasformula = argssInfo.get_NameVF(DMGRSIGNALSSHEETLIB.tlPropertyType.tlPropertyFormula);
                                        if (hasformula != null)
                                        {
                                            argssInfo.set_NameVF(DMGRSIGNALSSHEETLIB.tlPropertyType.tlPropertyFormula, null);
                                        }
                                        argssInfo.Name = kvp.Value;
                                    }
                                }
                                argColumn++;
                            }
                        }
                        else
                        {
                            throw new ArgumentException("Instrument Type doesn't match");
                        }
                    }
                    else
                    {
                        /* Create new pin information and attach to the existing signal 
                        and add as a new row after the existing signal's row.*/
                        pinInfo = signalInfo.Pins.CreateItem();
                        pinInfo.Name = nspcpObj.pinName;
                        pinInfo.Insttype = nspcpObj.propLogicalInstrument;
                        pinInfo.Wavedef = nspcpObj.NameValuePairs[SIGNAL_WAVE_DEFINITION_TYPE];
                        pinInfo.Sigtype = nspcpObj.NameValuePairs[SIGNAL_TYPE];
                        // signal's pin collection is 1-based; the object is inserted BEFORE this index
                        int PinIndex = signalInfo.Pins.Count;
                        signalInfo.Pins.Add(pinInfo, PinIndex);

                        argssInfo = pinInfo.Argss.CreateItem();
                        key.Trim();
                        foreach (String s in nspcpObj.NameValuePairs.Keys)
                        {
                            if (s.Equals(SIGNAL_TYPE) || s.Equals(SIGNAL_WAVE_DEFINITION_TYPE))
                                continue;
                            key = key + "," + s;
                        }
                        argssInfo.Name = key.TrimStart(',');
                        pinInfo.Argss.AddElement(argssInfo, LIST_OF_ALL_PARAMS);

                        foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                        {
                            if (kvp.Key.Equals(SIGNAL_TYPE) || kvp.Key.Equals(SIGNAL_WAVE_DEFINITION_TYPE))
                                continue;
                            argssInfo = pinInfo.Argss.CreateItem();
                            argssInfo.Name = kvp.Value;
                            pinInfo.Argss.Add(argssInfo, argColumn);
                            argColumn++;
                        }
                    }
                }
                else
                {
                    // Create New Signal And add at the end of the Signal sheet
                    DMSignalssignals NewSignalInfo;

                    NewSignalInfo = signalSheet.Signalss.CreateItem();
                    NewSignalInfo.Name = nspcpObj.propName;

                    pinInfo = NewSignalInfo.Pins.CreateItem();
                    pinInfo.Name = nspcpObj.pinName;
                    pinInfo.Insttype = nspcpObj.propLogicalInstrument;
                    pinInfo.Wavedef = nspcpObj.NameValuePairs[SIGNAL_WAVE_DEFINITION_TYPE];
                    pinInfo.Sigtype = nspcpObj.NameValuePairs[SIGNAL_TYPE];

                    NewSignalInfo.Pins.AddElement(pinInfo, NewSignalInfo.ElementIndex);
                    signalSheet.Signalss.Add(NewSignalInfo, signalSheet.Signalss.Count + 1);
                    argssInfo = pinInfo.Argss.CreateItem();
                    key.Trim();
                    foreach (String s in nspcpObj.NameValuePairs.Keys)
                    {
                        if (s.Equals(SIGNAL_TYPE) || s.Equals(SIGNAL_WAVE_DEFINITION_TYPE))
                            continue;
                        key = key + "," + s;
                    }
                    argssInfo.Name = key.TrimStart(',');
                    pinInfo.Argss.AddElement(argssInfo, LIST_OF_ALL_PARAMS);

                    foreach (KeyValuePair<String, String> kvp in nspcpObj.NameValuePairs)
                    {
                        if (kvp.Key.Equals(SIGNAL_TYPE) || kvp.Key.Equals(SIGNAL_WAVE_DEFINITION_TYPE))
                            continue;
                        argssInfo = pinInfo.Argss.CreateItem();
                        argssInfo.Name = kvp.Value;
                        pinInfo.Argss.Add(argssInfo, argColumn);
                        argColumn++;
                    }
                }
            }
            finally
            {
                // restore sheet update state
                if (workbook != null)
                    workbook.ScreenUpdateEnabled = updateState;
            }
        }

        /// <summary>
        /// The ReadSheet() method is used to read the signal's definition from signal sheet 
        /// and populate the read data into an NspcpItem object and returns the same.
        /// It will perform the above functionality under the condition that, 
        /// if the Passed SignalName(NspcpName), pinName and instrumentName are matching with
        /// one of the existing entries in the signal sheet.If The signal sheet which we want
        /// to read from, does not exist then, it throws an InteropServices.COMException.
        /// if any of the Passed Parameters are not matching with any of the Sheet's signal entries
        /// then this function returns null object. If Passed parameters contain null values, then
        /// The read function throws NullReferenceException.
        /// 
        /// </summary>
        /// <param name="pinName">pin name for which we want to read definition</param>
        /// <param name="instrName">Instrument name of which we want to read definition</param>
        /// <param name="nspcpName">Signal name of which we want to read definition from sheet</param>
        /// <returns>NspcpItem Object: This contains the Signal's definiton if the conditions 
        /// have been satisfied. Otherwise it contains null. </returns>
        public NspcpItem ReadSheet(String pinName, String instrName, String nspcpName)
        {
            //Declare signalSheets to get the list of Signal sheets available
            IDMSheets signalSheets;

            String[] paramNames;
            char[] delim = { ',' };
            DMSignalsSheet signalSheet = null;
            DMSignalssignals signalInfo;
            DMSignalssignalspin pinInfo = null;
            DMSignalssignalspinargs argsInfo;
            String paramValue;
            String signalType;
            String waveDefType;
            String signalNamesStr;
            String[] sigNames;
            String[] pinNames;
            String pinNamesStr;



            if ((pinName == null) || (instrName == null) || (nspcpName == null))
                throw new ArgumentNullException(" pinName Or instrName Or Signal are null");

            //Decompose the pin list that we got.
            String inputPinList = String.Join(",", DecomposePinList(pinName)).ToLower(); ;
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            inputPins.set_TSL(",", inputPinList);

            //// Following Line throws COM exception if Sheet not present.
            //signalSheet = (DMSignalsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
            signalSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);

            foreach (DMGRLib.IDMSheet item in signalSheets)
            {
                DMSignalsSheet signalSheetTemp = item as DMSignalsSheet;
                if (signalSheetTemp != null)
                {
                    signalSheet = signalSheetTemp;
                    break;
                }
            }

            if (signalSheet == null) throw new SheetNotPresentException("There are no active Signal Sheets present in the workbook.");//Throw exception if sheet is null. 

            signalNamesStr = signalSheet.Signalss.Names.CSL;
            sigNames = signalNamesStr.Split(delim);

            // case insensitive search of NspcpName
            for (int parse = FIRST_SIGNAL_NAME; parse < sigNames.Length; parse++)
            {
                if (nspcpName.ToLower().Equals(sigNames[parse].ToLower()))
                {
                    nspcpName = sigNames[parse];
                    break;
                }
            }
            signalInfo = (DMSignalssignals)signalSheet.Signalss.Find(nspcpName); // Throws COM exception if specified Nspcp ot found

            pinNamesStr = signalInfo.Pins.Names.CSL;

            String sheetPinNames = String.Join(",", DecomposePinList(pinNamesStr)).ToLower();
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings readPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            readPins.set_TSL(",", sheetPinNames);

            pinNames = pinNamesStr.Split(delim);

            if (!pinNamesStr.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = signalInfo.Pins.Find(pinNamesStr);
            }
            else if (pinName.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = signalInfo.Pins.Find(pinName); // Throws an COM Exception if Specified Pin not found
            }
            else if (inputPins.CSL.ToLower().Equals(readPins.CSL.ToLower()))
            {
                pinInfo = signalInfo.Pins.Find(inputPins.CSL);
            }
            else
            {
                for (int parse = FIRST_SIGNAL_NAME; parse < pinNames.Length; parse++)
                {
                    if (pinName.ToLower().Equals(pinNames[parse].ToLower()))
                    {
                        pinName = pinNames[parse];
                        pinInfo = signalInfo.Pins.Find(pinName);
                        break;
                    }
                }
            }
            if (pinInfo == null)
            {
                throw new ArgumentException("Invalid pin name Passed");
            }


            if ((nspcpName.ToLower().Equals(signalInfo.Name.ToLower())) &&
                ((pinName.ToLower().Equals(pinInfo.Name.ToLower())) ||
                (inputPins.CSL.ToLower().Equals(pinInfo.Name.ToLower())) ||
                (pinNamesStr.ToLower().Equals(pinInfo.Name.ToLower()))) &&
                (instrName.ToLower().Equals(pinInfo.Insttype.ToLower())))
            {
                signalType = pinInfo.Sigtype;
                waveDefType = pinInfo.Wavedef;
                argsInfo = (DMSignalssignalspinargs)pinInfo.Argss.get_Element(LIST_OF_ALL_PARAMS);// will throw an NullReferenceException if it fails
                paramValue = argsInfo.Name;
                paramNames = paramValue.Split(delim);
                Dictionary<String, String> keyValuePairs = new Dictionary<String, String>();
                keyValuePairs.Add(SIGNAL_TYPE, signalType);
                keyValuePairs.Add(SIGNAL_WAVE_DEFINITION_TYPE, waveDefType);
                for (int pF = FIRST_ARG_IN_SHEET; pF <= paramNames.Length + 1; pF++)
                {
                    argsInfo = (DMSignalssignalspinargs)pinInfo.Argss.get_Element(pF);
                    paramValue = argsInfo.Name;
                    keyValuePairs.Add(paramNames[pF - FIRST_ARG_IN_SHEET], paramValue);
                }
                return new NspcpItem(GlobalUtils.NspcpType.signal, nspcpName.ToLower(), pinName, pinInfo.Insttype, pinInfo.Insttype, keyValuePairs.Count, keyValuePairs);
            }
            else
            {
                throw new ArgumentException("Invalid Arguments Passed");
            }
        }


        /// <summary>
        /// The GetList() function is used to get the list of Signals from the Signal sheet
        /// which have been associated to the pinName and instrumentName which we are passing.
        /// </summary>
        /// <param name="pinName">Name of the pin for which we want to get the list of signals that are 
        /// associated to, if the Signal is also associated to Instrument of Name instrName</param>
        /// <param name="instrName">Name of the instrument for which we want to get the list of signals 
        /// that are associated to, if the Signal is also associated to the Pin of name pinName</param>
        /// <returns>An Array Of Sgnal names</returns>
        public String[] GetList(String pinName, String instrName)
        {
            DMSignalsSheet signalSheet;
            String pinInfo;
            
            //Declare signalSheets to get the list of Signals sheets available
            IDMSheets signalSheets;

            if ((pinName == null) || (instrName == null))
                throw new ArgumentNullException("pinName or instrName are null");

            //Decompose the pin list that we got.
            String inputPinList = String.Join(",", DecomposePinList(pinName)).ToLower(); ;
            PARAMETERCOLLECTIONINTERFACELib.ITCOStrings inputPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
            inputPins.set_TSL(",", inputPinList);

            ArrayList signalList = new ArrayList();

            try
            {
                // Following throws an exception if signal Sheet is not available
                //Get the list of Signals sheets by using FindSheetsByType         
                //signalSheet = (DMSignalsSheet)MyServices.DmgrServiceCOM.ContextMgr.FindSheetByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);           
                signalSheets = (IDMSheets)MyServices.DmgrServiceCOM.ContextMgr.FindSheetsByType(GlobalUtils.DMGR_SHEET_TYPE_SIGNALSSHEET);
                if ((signalSheets == null)||(signalSheets.Count == 0)) throw new ArgumentException(); //Throw exception if sheet is null.
                    //Loop through each Signal sheet
            foreach (DMGRLib.IDMSheet sheet in signalSheets)
            {
                    signalSheet = (DMSignalsSheet)sheet;
                    for (int signalId = FIRST_SIGNAL; signalId <= signalSheet.Signalss.Count; signalId++)
                    {
                        for (int pinId = FIRST_PIN_NAME; pinId <= signalSheet.Signalss[signalId].Pins.Count; pinId++)
                        {
                            //make sure the instruments match
                            if (instrName.ToLower().Equals(signalSheet.Signalss[signalId].Pins[pinId].Insttype.ToLower()))
                            {
                                //now check the pins match
                                pinInfo = signalSheet.Signalss[signalId].Pins[pinId].Name; // Throws COM exception if invalid pin is specified.
                                //Decompose this Row's pin list
                                String thisRowPinList = String.Join(",", DecomposePinList(pinInfo)).ToLower(); ;
                                PARAMETERCOLLECTIONINTERFACELib.ITCOStrings thisRowPins = (PARAMETERCOLLECTIONINTERFACELib.ITCOStrings)new TERCOMMONOBJLib.TCOStrings();
                                thisRowPins.set_TSL(",", thisRowPinList);
                                if (inputPins.Count == thisRowPins.Count)//if its not the same number of pins than it does not match
                                {
                                    Boolean match = true;
                                    //iterate through the input pins and make sure they all exist in this row
                                    for (int pinIndex = 1; pinIndex <= inputPins.Count; pinIndex++)
                                    {
                                        if (!thisRowPins.Has(inputPins[pinIndex]))
                                        {
                                            match = false;
                                            break;
                                        }

                                    }
                                    if (match)
                                        signalList.Add(signalSheet.Signalss[signalId].Name.ToLower());
                                }
                            }
                        }
                    }
                }
            }
            catch (System.ArgumentException)
            {
                throw new SheetNotPresentException("There are no active Signals Sheets present in the workbook.");
            }
            
                return (String[])signalList.ToArray(typeof(String));
        }
        /// <summary>
        /// Decomposes a Pin Spec into a list of pin names that compose that pinspec.
        /// </summary>
        /// <param name="pinspec">PinSpec Name</param>
        /// <returns>List of pins</returns>
        public String[] DecomposePinList(String pinspec)
        {

            if (pinspec == null) throw new ArgumentNullException("One of more arguments passed to DecomposePinList is null");
            Array arrPinList;
            String[] strPinList;
            int pinCount;

            //TypeCast Array to String[] so that it can be passed as a parameter
            //to the DecomposePinList method.
            strPinList = new String[0];
            arrPinList = strPinList as Array;

            try
            {
                if (m_signalHelper.IsValidPinSpec(pinspec))
                    //Decompose the pin spec to get the list of pins.
                    MyServices.DmgrServiceCOM.DecomposePinList(pinspec, out arrPinList, out pinCount);
            }
            catch (Exception e)
            {
                throw new ServicesException("Error in DMGR Service while Decomposing Pin List" + e.Message);
            }

            return ((String[])arrPinList);
        }
        #endregion
    }
}
