// ------------------------------------------------------------------
// (c) Teradyne, Inc, 2006
//     All Rights Reserved
// Inclusion of a copyright notice does not imply that this software
// has been published.  This software is the trade secret information
// of Teradyne, Inc.  Use of this software is only in accordance with
// the terms of a license agreement from Teradyne, Inc.
// ------------------------------------------------------------------
// UndoRedoManager.cs - Represents a framework to easily add undo / redo functionality
// *************************************************************************************/
//   Revision history:
//   Author         Date        Description of change
//   =================================================================
//   shoban         9/4/2007    Created
//   shoban         9/7/2007    Added support for not adding context if there are no
//                              undo information in a composite
// *************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace Teradyne.Igxl.Utilities
{
    /// <summary>
    /// Represents a framework that is resposible for managing undo / redo information. 
    /// 
    /// Users call StoreState to store the state to revert back later periodically.
    /// <example>
    /// OnCellChange()
    /// {
    ///   // validate data
    ///   ...
    ///   SaveState(observer, memento);
    /// }
    /// </example>
    /// 
    /// Note: The memento should maintain the current as well as the previous state
    /// as either will be required during undo or redo.
    /// 
    /// You can revert to the previous state or move to the last undone state by calling
    /// Undo or Redo respectively.
    /// 
    /// <example>
    /// OnUndoButtonClicked()
    /// {
    ///   undoManager.Undo();
    /// }
    /// </example>
    /// 
    /// This class can maintain undo information limited by specified capacity where 
    /// older data beyond capacity is discarded.
    /// 
    /// Undo states can be nested or grouped by means of the BeginContext / EndContext 
    /// methods. Grouped context can be undone as a single operation.
    /// 
    /// <example>
    /// OnCellChange()
    /// {
    ///   // validate data
    ///   ...
    ///   SaveState(observer, memento);
    /// }
    /// 
    /// OnCellPaste()
    /// {
    ///   BeginContext();
    ///   // for each cell, update the data. This will call OnCellChange
    ///   EndContext();
    /// }
    /// </example>
    /// 
    /// </summary>
    public class UndoRedoManager
    {
        // This is a list rather than a stack so that we can maintain limits on 
        // capacity and discard older undo actions
        private LinkedList<IUndoContext> m_undoList;
        private Stack<IUndoContext> m_redoQ;
        private bool m_ignoreStateChange;
        private int m_capacity;

        private CompositeUndoContext m_currentContext;

        /// <summary>
        /// Creates the undo manager. 
        /// </summary>
        /// <param name="capacity">Incicates the amount of undo information to maintain. 
        /// Composite undo contexts are counted as one</param>
        public UndoRedoManager(int capacity)
        {
            if (capacity < 1)
                throw new ArgumentOutOfRangeException("capacity", "Should be positive and greater than 1");

            m_undoList = new LinkedList<IUndoContext>();
            m_redoQ = new Stack<IUndoContext>();
            m_ignoreStateChange = false;

            m_capacity = capacity;

            m_currentContext = null;
        }

        /// <summary>
        /// Sets the control / model to the previous state
        /// </summary>
        /// <returns>true on success</returns>
        public bool Undo()
        {
            if (CanUndo)
            {
                IUndoContext currentContext = m_undoList.First.Value;
                
                if (ResetState(currentContext, true))
                {
                    m_undoList.RemoveFirst();
                    m_redoQ.Push(currentContext);
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Sets the control / model to the next redoable state
        /// </summary>
        /// <returns>Returns true of success</returns>
        public bool Redo()
        {
            if (CanRedo)
            {
                IUndoContext c = m_redoQ.Pop();
                if (ResetState(c, false))
                {
                    m_undoList.AddFirst(c);
                    return true;
                }
                else
                {
                    m_redoQ.Push(c);
                }
            }
            return false;
        }

        /// <summary>
        /// Undos all pending actions
        /// </summary>
        /// <returns>Returns true on success</returns>
        public bool UndoAll()
        {
            while (CanUndo)
            {
                if (!Undo())
                    return false;
            }

            return true;
        }

        /// <summary>
        /// Redos all pending action
        /// </summary>
        /// <returns>Returns true on success</returns>
        public bool RedoAll()
        {
            while (CanRedo)
            {
                if (!Redo())
                    return false;
            }

            return true;
        }

        /// <summary>
        /// Saves the current state of the system. This will be called whenever some data item changes that needs to be undone
        /// </summary>
        /// <param name="handler">The observer who will be notified when undo or redo for this state happens</param>
        /// <param name="currentState">The undo information which should contain the previous and current state</param>
        /// <returns>true on success</returns>
        public bool SaveState(IUndoable handler, Object currentState)
        {
            if (handler == null)
                throw new ArgumentNullException("handler", "Require a valid non-null IUndoable handler");

            if (Undoing) // An undo is in progress. We will not store state now
                return false;

            AddContext(new UndoContext(handler, currentState));

            if (CanRedo)
            {
                // Note: If we are adding new states while we have a few items to be redone,
                // redo will not make any sense anymore. So, we clear it off.
                m_redoQ.Clear();
            }

            if (m_undoList.Count + m_redoQ.Count > m_capacity)
            {
                // Discard the last item. We need not check redoQ as it will be empty
                m_undoList.RemoveLast();
            }

            return true;
        }

        private void AddContext(IUndoContext context)
        {
            if (m_currentContext != null)
                m_currentContext.Add(context);
            else
                m_undoList.AddFirst(context);
        }

        /// <summary>
        /// Clears all undo / redo information
        /// </summary>
        public void Clear()
        {
            m_undoList.Clear();
            m_redoQ.Clear();
        }

        /// <summary>
        /// Indicates if undo can be performed. Can be used by visual indicators 
        /// (enabling / disabling toolbar buttons, menu items etc)
        /// </summary>
        public bool CanUndo
        {
            get { return m_undoList.Count > 0; }
        }

        /// <summary>
        /// Indicates if redo can be performed. Can be used by visual indicators 
        /// (enabling / disabling toolbar buttons, menu items etc)
        /// </summary>
        public bool CanRedo
        {
            get { return m_redoQ.Count != 0; }
        }

        /// <summary>
        /// Ingore save state requests. There may be some stages where we do not wish to 
        /// store undo information (like initialization or system updates)
        /// </summary>
        public bool TrackChanges
        {
            get { return !m_ignoreStateChange; }
            set { m_ignoreStateChange = !value; }
        }

        /// <summary>
        /// Indicates that an undo / redo is in progress. 
        /// </summary>
        public bool Undoing
        {
            get { return m_ignoreStateChange; }
        }

        /// <summary>
        /// Indicates a complex operation is going to take place and that all undo information
        /// gathered will be grouped together into a single context. This should be called with a 
        /// matching EndContext. Begin / EndContext pairs can be nested.
        /// </summary>
        public void BeginContext()
        {
            m_currentContext = new CompositeUndoContext(m_currentContext);
        }

        /// <summary>
        /// Indicates that the current context can be closed which means all operations since 
        /// BeginContext will be clubbed together as a single undoable action.
        /// </summary>
        public void EndContext()
        {
            if (m_currentContext == null)
                throw new ApplicationException("EndContext called without call to BeginContext");

            IUndoContext c = m_currentContext;
            m_currentContext = m_currentContext.Parent as CompositeUndoContext;

            // In case a composite context does not have any undo information, 
            // discard it
            if(c.Count > 0)
                AddContext(c); // save the current set of undos into the undo list
        }

        private bool ResetState(IUndoContext c, bool isUndoing)
        {
            m_ignoreStateChange = true;

            bool ret = c.SetState(isUndoing);

            m_ignoreStateChange = false;

            return ret;
        }
    }
}
