
/* -----------------------------------------------------------------
 * (c) Teradyne, Inc, 2007
 * All Rights Reserved
 * Inclusion of a copyright notice does not imply that this software
 * has been published.  This software is the trade secret information
 * of Teradyne, Inc.  Use of this software is only in accordance with
 * the terms of a license agreement from Teradyne, Inc.
 * -------------------------------------------------------------------
 */
/*-------------------------------------------------------------------------------
 * frmPinsGroupsSelector.cs is responsible for providing a user interface for the
 * user to select pins/pingroups or create pin lists based on the pins present in
 * the pinmap. Users can also filter their selections based on the chosen logical
 * instrument.
 * ------------------------------------------------------------------------------
 */
/*
 ***************************************************************************************
 *   Revision history:
 *   Author          Date        Description of change
 *   =================================================================
 *  Shoban Jayaraj  01/03/08    tersw00115705 - regexp using ',' operator lists incorrect pins
 *  Suresh V A      12/07       Defect Fix tersw00114810
 *	Palaniappa Babu 11/23		tersw00114456
 *  Shoban Jayaraj  11/22       tersw00114461 & tersw00114335
 *  Andy Kittross   07-Nov-07   Optimizations (tersw00113622)
 *  Kiran           11/10/07    Fixed the defect : tersw00112608
 *  Nadeem Khan     8/12        Created.
 *  Shoban Jayaraj  9/14        Fixes for tersw00111273, tersw00111272 and tersw00111271
 *  Shoban Jayaraj  9/19        Fix for tersw00111518 - sorting of pin group list view 
 *                              columns
 *  Shoban Jayaraj  9/21        tersw00111681 - filters in pin / group selector
 *  Shoban Jayaraj  9/22        tersw00111559 - Prompt when user cancels with changes made
 *  Shoban Jayaraj  9/26        tersw00111908 - Allow user to click on 'ok' even if there
 *                              are no selections. This will allow user to clear existing
 *                              selection.
 *  Palaniappa Babu 9/26        tersw00111914 - Pin group selector Discard Message
 * Abdul Rajak Shaik 10/1       Fix for tersw00112022
 * Andy Kittross    10/4        Fix tersw00112333 (move up/down not working) by attaching sorter objects 
 *                              when headers are clicked, and then detaching them so up/down buttons work
 * Maria Raykin     10/5        Fixed pin "case" problem (tersw00112398)
 ****************************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Text.RegularExpressions;
using System.Runtime.InteropServices;

namespace Teradyne.Igxl.PSetSignalsEditor
{
    public partial class frmPinsGroupsSelector : Form
    {
        String m_finalList;         //Variable storing the final list of pins selected.
        String m_instrName;         //The instrument selected by user in the main editor window
        InstrumentEngine m_InstrEngine;
        Dictionary<String, String> m_pinTable;
        String[] m_existPinList = null;
        Boolean m_FetchOperation;
        Boolean m_validExistingPinList = false;

        // To allow for sorting in the list view
        ListViewColumnSorter m_selectedListSorter;
        ListViewColumnSorter m_availableListSorter;

        const String CHANNEL_FILTER_ALL = "<All>";
        // Indicates that columns in list view will be resized taking into account, the header
        // and the contents present under the particular column
        const int LVSCW_AUTOSIZE_USEHEADER = -2; 

        /// <summary>
        /// Property to set or get the final list of pins selected.
        /// </summary>
        public String PinGroupList 
        {
            get
            {
                return m_finalList;
            }

            set
            {
                m_finalList = value;
            }

        }
        
        /// <summary>
        /// Constructor which initializes all references passed and creates the Pin to Instrument Mapping Table.
        /// </summary>
        /// <param name="coreHandle">Reference to teh </param>
        /// <param name="instrEngine"></param>
        /// <param name="instrName"></param>
        /// <param name="existingList">Existing list of pins if any</param>
        public frmPinsGroupsSelector(ref InstrumentEngine instrEngine, String instrName, String existingList)
        {

            if(instrEngine==null||instrName==null||existingList==null)  throw new ArgumentNullException("One or more arguments passed to the Pin Selector are null");

            InitializeComponent();
            m_InstrEngine = instrEngine;
            m_instrName = instrName;
            m_FetchOperation = false;

            //Check if the list passed is valid
            if ((existingList != null) && (!existingList.Equals(System.String.Empty)))
            {
                m_validExistingPinList = m_InstrEngine.CheckPin(ref existingList);
                // fix "case" problem (tersw00112398)
                m_existPinList = existingList.Split(',');
            }
            else
            {
                m_validExistingPinList = false;
                m_existPinList = null;
            }

            //Get the pin->instrument map.
            m_pinTable = m_InstrEngine.CreatePinTable();

            // Sorters for the list view
            m_availableListSorter = new ListViewColumnSorter();
            m_availableListSorter.Order = SortOrder.Ascending;
            m_selectedListSorter = new ListViewColumnSorter();
            m_selectedListSorter.Order = SortOrder.Ascending;
            // But we don't attach any sorter unless the column header is clicked, since we have up/down buttons as well
            lvwAvailable.ListViewItemSorter = null;
            lvwAvailable.Sorting = SortOrder.None;
            lvwSelected.ListViewItemSorter = null;
            lvwSelected.Sorting = SortOrder.None;

            Cursor.Current = Cursors.Default;

            


        }




        #region Check the Vertical bar is visible or not, in the ListViewControl 

        private const int WS_HSCROLL = 0x100000;
        private const int WS_VSCROLL = 0x200000;
        private const int GWL_STYLE = (-16);


        [DllImport("user32", CharSet = CharSet.Auto)]
        private static extern int GetWindowLong(IntPtr hwnd, int nIndex);

       


        internal  bool IsVerticalScrollBarVisible(Control ctrl)
        {
            if (!ctrl.IsHandleCreated)
                return false;


            return (GetWindowLong(ctrl.Handle, GWL_STYLE) & WS_VSCROLL) != 0;
        }

        /// <summary>
        /// This method is used to resize the column
        /// </summary>
        private void AdjustColumnSize()
        {
            ///****************************************************************************************/
            ///Set the width of the columns as -2, for both ListView Control column, for fixing the 
            ///defect tersw00114810
            lvwAvailable.Columns[0].Width = -2;
            lvwAvailable.Columns[1].Width = -2;

            lvwSelected.Columns[0].Width = -2;
            lvwSelected.Columns[1].Width = -2;
            

            if (IsVerticalScrollBarVisible((Control)lvwAvailable))
            {
                // '-21'for sets the rightmost column width to allow enough space for a vertical scrollbar to appear
                lvwAvailable.Columns[1].Width = lvwAvailable.Width - lvwAvailable.Columns[0].Width - 21;
            }
            else
            {
                //Verical bar is not visible then we
                lvwAvailable.Columns[1].Width = lvwAvailable.Width - lvwAvailable.Columns[0].Width - 4;
            }

            if (IsVerticalScrollBarVisible((Control)lvwSelected))
            {
                // '-21'for sets the rightmost column width to allow enough space for a vertical scrollbar to appear
                lvwSelected.Columns[1].Width = lvwSelected.Width - lvwSelected.Columns[0].Width - 21;
            }
            else
            {
                //Verical bar is not visible then we
                lvwSelected.Columns[1].Width = lvwSelected.Width - lvwSelected.Columns[0].Width - 4;
            }


            ///******************************************************************************************/
        }

        #endregion 

        /// <summary>
        /// Handler for the change in the Available List.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstAvailable_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateButtons();
            AdjustColumnSize();
        }

        /// <summary>
        /// Handler for the change in moving elements from Available to Selected List.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveRight_Click(object sender, EventArgs e)
        {
            int i = 0;
            foreach (ListViewItem lItem in lvwSelected.Items)
            {
                lvwSelected.Items[i].Selected = false;
                i++;
            }
            foreach (ListViewItem LItem in lvwAvailable.SelectedItems)
            {
                lvwAvailable.Items.Remove(LItem);
                lvwSelected.Items.Add(LItem).Selected = true;
            }

                UpdateButtons();
                AdjustColumnSize();
            
        }

        /// <summary>
        /// Handler for the change in moving all elements from Available to Selected List.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveAllRight_Click(object sender, EventArgs e)
        {

            foreach (ListViewItem LItem in lvwAvailable.Items)
            {
                lvwAvailable.Items.Remove(LItem);
                lvwSelected.Items.Add(LItem);
            }
            UpdateButtons();
            AdjustColumnSize();
        }


        /// <summary>
        /// Handler for the change in moving elements from Selected to Available List.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveLeft_Click(object sender, EventArgs e)
        {
            int i = 0;
            foreach (ListViewItem lItem in lvwAvailable.Items)
            {
                lvwAvailable.Items[i].Selected = false;
                i++;
            }
            foreach (ListViewItem LItem in lvwSelected.SelectedItems)
            {
                lvwSelected.Items.Remove(LItem);
                lvwAvailable.Items.Add(LItem);
            }

            UpdateButtons();
            AdjustColumnSize();
            
        }

        /// <summary>
        /// Handler for the change in moving all elements from Selected to Available List.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveAllLeft_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem LItem in lvwSelected.Items)
            {
                lvwSelected.Items.Remove(LItem);
                lvwAvailable.Items.Add(LItem);
            }

            UpdateButtons();
            AdjustColumnSize();
        }

        /// <summary>
        /// Handler for the change in moving up elements in the selected list.
        /// This is only invoked if one element is selected to be moved up.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            // swap places
            int idx = lvwSelected.SelectedItems[0].Index;
            ListViewItem item1 = lvwSelected.Items[idx];
            ListViewItem item2 = lvwSelected.Items[idx - 1];
            lvwSelected.Items.RemoveAt(idx);
            lvwSelected.Items.RemoveAt(idx - 1);
            lvwSelected.Items.Insert(idx - 1, item1);
            lvwSelected.Items.Insert(idx, item2);
            lvwSelected.Focus();
            lvwSelected.Items[idx].Selected = false;
            lvwSelected.Items[idx - 1].Selected = true;

            // Selected = true will not trigger an event change. So raise one manually
            lvwSelected_SelectedIndexChanged(null, null);
        }

        /// <summary>
        /// Handler for the change in moving down elements in the selected list.
        /// This is only invoked if one element is selected to be moved up.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            // swap places           
            int idx = lvwSelected.SelectedItems[0].Index;
            ListViewItem item1 = lvwSelected.Items[idx];
            ListViewItem item2 = lvwSelected.Items[idx + 1];
            lvwSelected.Items.RemoveAt(idx + 1);
            lvwSelected.Items.RemoveAt(idx);
            lvwSelected.Items.Insert(idx, item2);
            lvwSelected.Items.Insert(idx + 1, item1);
            lvwSelected.Focus();
            lvwSelected.Items[idx].Selected = false;
            lvwSelected.Items[idx + 1].Selected = true;

            // Selected = true will not trigger an event change. So raise one manually
            lvwSelected_SelectedIndexChanged(null, null);
        }



        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Create a comma separated list of pins based on elements in the selected list box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            System.Collections.ArrayList pinArray = new System.Collections.ArrayList ();
            if (lvwSelected.Items.Count > 0)
            {
                foreach (ListViewItem it in lvwSelected.Items)
                {
                    pinArray.Add(it.Text);
                }
                this.PinGroupList = String.Join(",", (string[])pinArray.ToArray(typeof(string)));
            }
            this.Close();
        }


        private void lvwSelected_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateButtons();
            AdjustColumnSize();
        }

        private void lvwAvailable_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateButtons();
            AdjustColumnSize();
        }

        private void btnFetch_Click(object sender, EventArgs e)
        {
            //Upon clicking the fetch button, we populate the pin list view.
            m_FetchOperation = true;
            PopulatePinListView();
        }

        private Regex GetProcessedRegExp(String rawRegExp)
        {
            String tmpRegEx = rawRegExp.Trim().Replace("*", ".*");
            tmpRegEx = tmpRegEx.Replace("?", ".?");
            tmpRegEx = tmpRegEx.Replace(" ", "");
            tmpRegEx = tmpRegEx.Replace(",", "|");

            Regex r;

            try
            {
                r = new Regex(String.Format("^({0})$", tmpRegEx), RegexOptions.IgnoreCase);
            }
            catch (Exception)
            {
                // User has provided an invalid regexp
                return null;
            }

            return r;
        }

        /// <summary>
        /// This procedure poplates the available list with information.
        /// </summary>
        private void PopulatePinListView()
        {
            //Add any list of items in selected list (if already in the pin list section)
            if(m_validExistingPinList && (m_existPinList!=null) && !m_FetchOperation)
            {
                //If there are a some valid pins already listed in the text box of the UI,
                //add them to the selected list.
                //However, we do not do this incase of a fetch operation.
                foreach (string pin in m_existPinList)
                {
                    ListViewItem item = new ListViewItem(pin);

                    //Attach the associated instrument list.
                    if(m_pinTable.ContainsKey(pin)) {
                        String instruments = m_pinTable[pin];
                        item.SubItems.Add(instruments);
                    }

                    if (!AlreadySelected(pin))
                    {                       
                        lvwSelected.Items.Add(item);
                        UpdateButtons();
                        AdjustColumnSize();
                    }
                }
            }

            lvwAvailable.Items.Clear();

            // Setup the regexp
            if(txtFilter.Text.Trim().Equals(""))
                txtFilter.Text = "*"; // We do not want an empty filter

            Regex regexp = GetProcessedRegExp(txtFilter.Text);
            if (regexp == null)
            {
                txtFilter.Text = "*";
                MessageBox.Show(Properties.Resources.InvalidPinFilter, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Iterate through the Pin Table.
            foreach (KeyValuePair<String, String> kvp in m_pinTable)
            {
                //See if the pin matches the pin filter specified in txtFilter
                if (!regexp.IsMatch(kvp.Key))
                    continue;

                //Get the list of instruments corresponding to a pin.
                String[] instruments = kvp.Value.Split(new char[] { ',' });
                if (instruments.Length > 0)
                {
                    String selectedInstr = cboChannelTypeFilter.Text.Trim();
                    if (selectedInstr.Equals("") || selectedInstr.Equals(CHANNEL_FILTER_ALL))    //If user has not selected an instrument filter, add all items
                    {
                        ListViewItem item = new ListViewItem(kvp.Key, 0);
                        item.SubItems.Add(kvp.Value);
                        //Check if the item is not already in the 'Selected List' pane first
                        if (!AlreadySelected(kvp.Key))
                        {
                            lvwAvailable.Items.Add(item);   //Add the item to the list of available elements.
                        }
                    }
                    else if (kvp.Value.ToLower().Contains(selectedInstr.ToLower()))  //If the user has selected an instrument filter then only add the pin if its instrument 
                    {                                                           //matches with the filter name.
                        ListViewItem item = new ListViewItem(kvp.Key, 0);
                        item.SubItems.Add(kvp.Value);
                        if (!AlreadySelected(kvp.Key))
                        {
                            lvwAvailable.Items.Add(item);
                        }
                    }
                }
            }
        }



        /// <summary>
        /// Checks whether element is already in selected list or not.
        /// </summary>
        /// <param name="p">item to check for</param>
        /// <returns>true if already a part of selected list.</returns>
        private bool AlreadySelected(string p)
        {
            foreach (ListViewItem item in lvwSelected.Items)
            {
                if (item.Text.Equals(p)) return true;
            }
            return false;
        }

       

        private void frmPinsGroupsSelector_Load(object sender, EventArgs e)
        {
            // Show the resizing handle at lower right of the form
            this.SizeGripStyle = SizeGripStyle.Show;

            // Set up display properties
            lvwAvailable.ShowItemToolTips = true;

            lvwSelected.ShowItemToolTips = true;

            //Set the instrument drop down to what the user has selected
            //in the main editor window.

            List<String> instList = new List<String>();
            instList.Add(m_instrName.Equals("") ? CHANNEL_FILTER_ALL : m_instrName);

            cboChannelTypeFilter.DataSource = instList;
            cboChannelTypeFilter.SelectedItem = 0; // select the first item

            //Fetch the default list of pins/pingroups
            PopulatePinListView();

            UpdateButtons();

            AdjustColumnSize();
        }


        /// <summary>
        /// This handler populates the Channel Type filter dropdown with all instrument names.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cboChannelTypeFilter_DropDown(object sender, EventArgs e)
        {
            try
            {
                String[] strInstrList = m_InstrEngine.GetInstruments("");  //Retrieve list of instrument names from Instrument Engine.
                List<String> instrList = new List<String>();
                instrList.Add(CHANNEL_FILTER_ALL);
                instrList.AddRange(strInstrList);
                cboChannelTypeFilter.DataSource = instrList;
            }
            catch (Exception exp)
            {
                Console.Write("Exception caught in cboChannelTypeFilter : {0}", exp.StackTrace);
            }
        }


      
        private void UpdateButtons()
        {
            // btnOK.Enabled = lvwSelected.Items.Count != 0; - We allow the user to clear previously selected item
            btnMoveAllRight.Enabled = lvwAvailable.Items.Count != 0;
            btnMoveAllLeft.Enabled = lvwSelected.Items.Count != 0;
            btnMoveRight.Enabled = lvwAvailable.SelectedItems.Count != 0;
            btnMoveLeft.Enabled = lvwSelected.SelectedItems.Count != 0;

            btnMoveUp.Enabled = lvwSelected.SelectedItems.Count == 1 && lvwSelected.SelectedItems[0].Index != 0;
            btnMoveDown.Enabled = lvwSelected.SelectedItems.Count == 1 && lvwSelected.SelectedItems[0].Index != lvwSelected.Items.Count-1;

            lvwSelected.Columns[0].Width = LVSCW_AUTOSIZE_USEHEADER;
            lvwAvailable.Columns[0].Width = LVSCW_AUTOSIZE_USEHEADER;
        }

        /// <summary>
        /// Called when the column is clicked to sort by a specified column
        /// </summary>
        private void lvwAvailable_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Determine if clicked column is already the column that is being sorted.
            if (e.Column == m_availableListSorter.SortColumn)
            {
                m_availableListSorter.Order = m_availableListSorter.Order == SortOrder.Ascending ? SortOrder.Descending : SortOrder.Ascending;
            }
            else
            {
                // Set the column number that is to be sorted; default to ascending.
                m_availableListSorter.SortColumn = e.Column;
                m_availableListSorter.Order = SortOrder.Ascending;
            }

            // we only attach the sorter for the duration of the sort operation so that 
            // sorter will be detached if user clicks up/down buttons
            lvwAvailable.ListViewItemSorter = m_availableListSorter;
            lvwAvailable.Sort();
            lvwAvailable.ListViewItemSorter = null;
        }

        /// <summary>
        /// Called when the column is clicked to sort by a specified column
        /// </summary>
        private void lvwSelected_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Determine if clicked column is already the column that is being sorted.
            if (e.Column == m_selectedListSorter.SortColumn)
            {
                m_selectedListSorter.Order = m_selectedListSorter.Order == SortOrder.Ascending ? SortOrder.Descending : SortOrder.Ascending;
            }
            else
            {
                // Set the column number that is to be sorted; default to ascending.
                m_selectedListSorter.SortColumn = e.Column;
                m_selectedListSorter.Order = SortOrder.Ascending;
            }

            // we only attach the sorter for the duration of the sort operation so that 
            // sorter will be detached if user clicks up/down buttons
            lvwSelected.ListViewItemSorter = m_selectedListSorter;
            lvwSelected.Sort();
            lvwSelected.ListViewItemSorter = null;
        }

        private void lvwAvailable_DoubleClick(object sender, EventArgs e)
        {
            int i = 0;
            foreach (ListViewItem lItem in lvwSelected.Items)
            {
                lvwSelected.Items[i].Selected = false;
                i++;
            }
            foreach (ListViewItem LItem in lvwAvailable.SelectedItems)

            {
                lvwAvailable.Items.Remove(LItem);
                lvwSelected.Items.Add(LItem);
            }

            UpdateButtons();

            AdjustColumnSize();

        }

        private void lvwSelected_DoubleClick(object sender, EventArgs e)
        {
            int i = 0;
            foreach (ListViewItem lItem in lvwAvailable.Items)
            {
                lvwAvailable.Items[i].Selected = false;
                i++;
            }
            foreach (ListViewItem LItem in lvwSelected.SelectedItems)

            {
                lvwSelected.Items.Remove(LItem);
                lvwAvailable.Items.Add(LItem);
            }

            UpdateButtons();    

            AdjustColumnSize();
        }

        private void frmPinsGroupsSelector_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (PinGroupList != null)
                return; // User has commited the changes via the ok button

            if (PinGroupList == null && m_existPinList == null)
                return; // Both original and current selection is null

            if (!m_validExistingPinList && lvwSelected.Items.Count == 0)
                return; // There are no orignal valid pins and the user has not selected any

            if (lvwSelected.Items.Count != m_existPinList.Length)
            {
                // warn the user
                if (MessageBox.Show(Properties.Resources.PSetSignalEditorDiscardData, this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    e.Cancel = false;
                    return;
                }
                else
                {
                    e.Cancel = true;
                    return;
                }
            }

            foreach (String name in m_existPinList)
            {
                bool bFound = false;
                foreach (ListViewItem item in lvwSelected.Items)
                {
                    if (item.Text.Equals(name))
                    {
                        bFound = true;
                        break;
                    }
                }

                if (!bFound)
                {
                    // warn the user again
                    if (MessageBox.Show(Properties.Resources.PSetSignalEditorDiscardData  , this.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) == DialogResult.Cancel)
                    {
                        e.Cancel = true;
                    }

                    break; // We have asked the user. No need to continue checking again
                }
            }
        }

        // If Grid Control has focus and no List items are selected, select the first one
        private void lvwAvailable_Enter(object sender, EventArgs e)
        {
            if (lvwAvailable.Items.Count != 0)
            {
                if (lvwAvailable.SelectedItems.Count == 0)
                {
                    lvwAvailable.Items[0].Selected = true;
                }

            }
        }

        // If Grid Control has focus and no List items are selected, select the first one
        private void lvwSelected_Enter(object sender, EventArgs e)
        {
            if (lvwSelected.Items.Count != 0)
            {
                if (lvwSelected.SelectedItems.Count == 0)
                {
                    lvwSelected.Items[0].Selected = true;
                }

            }
        }

        private void txtFilter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                this.btnFetch.PerformClick();
            }
        }

        private void cboChannelTypeFilter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                this.btnFetch.PerformClick();
            }
        }

        private void frmPinsGroupsSelector_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                this.btnOK.PerformClick();
            }
        }

        private void lvwSelected_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                this.btnOK.PerformClick();
            }
        }

        private void lvwAvailable_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar.Equals('\r'))
            {
                this.btnOK.PerformClick();
            }
        }

      
    }
}